---------------------------------------------------------------------------------------------------
--	Project				: Universal Display Controller
--	File description	: Universal Display Controller AXI with DVI output wrapper
--	File name			: en_udc_axi_2d_dvi_xil_x7.vhd
--	Author				: Christoph Glattfelder
---------------------------------------------------------------------------------------------------
--	Copyright (c) 2014 by Enclustra GmbH, Switzerland
--	All rights reserved.
---------------------------------------------------------------------------------------------------
--	Notes:
--
---------------------------------------------------------------------------------------------------


---------------------------------------------------------------------------------------------------
-- libraries
---------------------------------------------------------------------------------------------------

library ieee;
	use ieee.std_logic_1164.all;
	use ieee.numeric_std.all;
	
library en_udc_axi_2d_dvi_xil_x7_cl;
	use en_udc_axi_2d_dvi_xil_x7_cl.en_cl_base_pkg.all;
    
library lib_en_udc_axi_2d_dvi_xil_x7;
	use lib_en_udc_axi_2d_dvi_xil_x7.en_udc_pkg.all;

---------------------------------------------------------------------------------------------------
-- entity declaration
---------------------------------------------------------------------------------------------------

entity en_udc_axi_2d_dvi_xil_x7 is
	generic (
		C_M_AXI_DATAWIDTH	: positive	:= 64;
		
		C_S_AXI_BASEADDR	: std_logic_vector	:= X"FFFFFFFF";
		C_S_AXI_HIGHADDR	: std_logic_vector	:= X"00000000";

		ColorWidth_g		: positive	:= 24;		-- 16 or 24
		MaxBurstLength_g	: positive	:= 128;
        
		EnableBrightness_g	: boolean	:= false;
		BrightPMWDiv_g		: positive	:= 1; 		-- Brightness PWM Clk divider        

		HResolution_g		: positive	:= 1024; 	-- pixel
		HSyncPulseWidth_g	: positive	:= 136; 	-- pixel
		HSyncPulsePol_g		: std_logic	:= '0'; 	-- 0=neg, 1=pos
		HFrontPorchLength_g	: natural	:= 24;		-- pixel
		HBackPorchLength_g	: natural	:= 160;		-- pixel
		
		VResolution_g		: positive	:= 768; 	-- lines
		VSyncPulseWidth_g	: positive	:= 6;		-- lines
		VSyncPulsePol_g		: std_logic	:= '1'; 	-- 0=neg, 1=pos
		VFrontPorchLength_g	: natural	:= 3;		-- lines
		VBackPorchLength_g	: natural	:= 29;		-- lines		
		
		BlackFramesPU_g		: natural	:= 2;		-- no of black frames at power up
		BlackFramesPD_g		: natural	:= 2;		-- no of black frames at power down
		
		RdFifoDepth_g		: positive	:= 2048;	-- Pixel
		RdFifoFillLevel_g	: positive	:= 1536; 
		
		UseAsyncFifos_g		: boolean	:= true;
		DispClkFreq_g		: positive	:= 65E6;	-- Hz
		M_AXI_ClkFreq_g		: positive	:= 100E6	-- Hz
	);
	port (
		-- global signals
		Clk					: in	std_logic;
		Rst					: in	std_logic;
	
		-- HDMI/DVI output
		DviClk_p			: out	std_logic;		
		DviClk_n			: out	std_logic;		
		DviData_p			: out	std_logic_vector(2 downto 0);
		DviData_n			: out	std_logic_vector(2 downto 0);
        
		-- Display signal
		Disp_Ena			: out	std_logic;
		Disp_Bright			: out	std_logic;        

		-- AXI4lite slave interface
		S_AXI_ACLK			: in	std_logic;
		S_AXI_ARESETN		: in	std_logic;
		S_AXI_AWADDR		: in	std_logic_vector(9 downto 0);
		S_AXI_AWVALID		: in	std_logic;
		S_AXI_WDATA			: in	std_logic_vector(31 downto 0);
		S_AXI_WSTRB			: in	std_logic_vector(3 downto 0);
		S_AXI_WVALID		: in	std_logic;
		S_AXI_BREADY		: in	std_logic;
		S_AXI_ARADDR		: in	std_logic_vector(9 downto 0);
		S_AXI_ARVALID		: in	std_logic;
		S_AXI_RREADY		: in	std_logic;
		S_AXI_ARREADY		: out	std_logic;
		S_AXI_RDATA			: out 	std_logic_vector(31 downto 0);
		S_AXI_RRESP			: out 	std_logic_vector(1 downto 0);
		S_AXI_RVALID		: out 	std_logic;
		S_AXI_WREADY		: out 	std_logic;
		S_AXI_BRESP			: out 	std_logic_vector(1 downto 0);
		S_AXI_BVALID		: out 	std_logic;
		S_AXI_AWREADY		: out 	std_logic;
		Irpt_N				: out	std_logic;
		
		-- AXI4 master interface (display)
		M_AXI_ACLK			: in	std_logic;
		M_AXI_ARESETN		: in	std_logic;
		M_AXI_AWADDR		: out	std_logic_vector(31 downto 0);
		M_AXI_AWVALID		: out	std_logic;
		M_AXI_AWREADY		: in	std_logic;
		M_AXI_AWLEN			: out	std_logic_vector(7 downto 0);
		M_AXI_AWSIZE		: out	std_logic_vector(2 downto 0);
		M_AXI_AWBURST		: out	std_logic_vector(1 downto 0);
		M_AXI_AWLOCK		: out	std_logic;
		M_AXI_AWCACHE		: out	std_logic_vector(3 downto 0);
		M_AXI_AWPROT		: out	std_logic_vector(2 downto 0);
		M_AXI_WDATA			: out	std_logic_vector(C_M_AXI_DATAWIDTH-1 downto 0);
		M_AXI_WSTRB			: out	std_logic_vector(C_M_AXI_DATAWIDTH/8-1 downto 0);
		M_AXI_WVALID		: out	std_logic;
		M_AXI_WREADY		: in	std_logic;
		M_AXI_WLAST			: out	std_logic;
		M_AXI_BRESP			: in	std_logic_vector(1 downto 0);
		M_AXI_BVALID		: in	std_logic;
		M_AXI_BREADY		: out	std_logic;
		M_AXI_ARADDR		: out	std_logic_vector(31 downto 0);
		M_AXI_ARVALID		: out	std_logic;
		M_AXI_ARREADY		: in	std_logic;
		M_AXI_ARLEN			: out	std_logic_vector(7 downto 0);
		M_AXI_ARSIZE		: out	std_logic_vector(2 downto 0);
		M_AXI_ARBURST		: out	std_logic_vector(1 downto 0);
		M_AXI_ARLOCK		: out	std_logic;
		M_AXI_ARCACHE		: out	std_logic_vector(3 downto 0);
		M_AXI_ARPROT		: out	std_logic_vector(2 downto 0);
		M_AXI_RDATA			: in	std_logic_vector(C_M_AXI_DATAWIDTH-1 downto 0);
		M_AXI_RVALID		: in	std_logic;
		M_AXI_RREADY		: out	std_logic;
		M_AXI_RRESP			: in	std_logic_vector(1 downto 0);
		M_AXI_RLAST			: in	std_logic;
		
		-- AXI4 master interface (2D accelerator)
		M2_AXI_ACLK			: in	std_logic;
		M2_AXI_ARESETN		: in	std_logic;
		M2_AXI_AWADDR		: out	std_logic_vector(31 downto 0);
		M2_AXI_AWVALID		: out	std_logic;
		M2_AXI_AWREADY		: in	std_logic;
		M2_AXI_AWLEN		: out	std_logic_vector(7 downto 0);
		M2_AXI_AWSIZE		: out	std_logic_vector(2 downto 0);
		M2_AXI_AWBURST		: out	std_logic_vector(1 downto 0);
		M2_AXI_AWLOCK		: out	std_logic;
		M2_AXI_AWCACHE		: out	std_logic_vector(3 downto 0);
		M2_AXI_AWPROT		: out	std_logic_vector(2 downto 0);
		M2_AXI_WDATA		: out	std_logic_vector(C_M_AXI_DATAWIDTH-1 downto 0);
		M2_AXI_WSTRB		: out	std_logic_vector(C_M_AXI_DATAWIDTH/8-1 downto 0);
		M2_AXI_WVALID		: out	std_logic;
		M2_AXI_WREADY		: in	std_logic;
		M2_AXI_WLAST		: out	std_logic;
		M2_AXI_BRESP		: in	std_logic_vector(1 downto 0);
		M2_AXI_BVALID		: in	std_logic;
		M2_AXI_BREADY		: out	std_logic;
		M2_AXI_ARADDR		: out	std_logic_vector(31 downto 0);
		M2_AXI_ARVALID		: out	std_logic;
		M2_AXI_ARREADY		: in	std_logic;
		M2_AXI_ARLEN		: out	std_logic_vector(7 downto 0);
		M2_AXI_ARSIZE		: out	std_logic_vector(2 downto 0);
		M2_AXI_ARBURST		: out	std_logic_vector(1 downto 0);
		M2_AXI_ARLOCK		: out	std_logic;
		M2_AXI_ARCACHE		: out	std_logic_vector(3 downto 0);
		M2_AXI_ARPROT		: out	std_logic_vector(2 downto 0);
		M2_AXI_RDATA		: in	std_logic_vector(C_M_AXI_DATAWIDTH-1 downto 0);
		M2_AXI_RVALID		: in	std_logic;
		M2_AXI_RREADY		: out	std_logic;
		M2_AXI_RRESP		: in	std_logic_vector(1 downto 0);
		M2_AXI_RLAST		: in	std_logic
	);
end entity en_udc_axi_2d_dvi_xil_x7;

---------------------------------------------------------------------------------------------------
-- architecture implementation
---------------------------------------------------------------------------------------------------

architecture rtl of en_udc_axi_2d_dvi_xil_x7 is

	constant ColorFrmt_c	: natural := choose(ColorWidth_g=16, 1, 2); -- 1=RBG 5:6:5, 2=RGB 8:8:8

	-- Display signal
	signal Disp_Rdy			: std_logic;
	signal Disp_VSync		: std_logic;
	signal Disp_HSync		: std_logic;
	signal Disp_DTMG		: std_logic;
	signal Disp_Data		: std_logic_vector (ColorWidth_g-1 downto 0);
	signal Dvi_Data			: std_logic_vector (23 downto 0);

begin

	-----------------------------------------------------------------------------------------------
	-- assertions
	-----------------------------------------------------------------------------------------------

	assert ColorWidth_g = 24 
		report "en_udc_axi_2d_dvi_xil_x7: ColorWidth_g = 24 is recommended for DVI output !!!" 
		severity warning;	
		
	-----------------------------------------------------------------------------------------------
	-- Display controller core
	-----------------------------------------------------------------------------------------------
	
	i_udc : entity lib_en_udc_axi_2d_dvi_xil_x7.en_udc_axi_2d
		generic map (
			Version_g			=> UdcCoreVers_c,
			
			C_M_AXI_DATAWIDTH	=> C_M_AXI_DATAWIDTH,
			C_S_AXI_BASEADDR	=> C_S_AXI_BASEADDR,
			C_S_AXI_HIGHADDR	=> C_S_AXI_HIGHADDR,
			
			PxlClkDivider_g		=> 1,
			ColorWidth_g		=> ColorWidth_g,
			ColorFrmt_g			=> ColorFrmt_c,	
			MaxBurstLength_g	=> MaxBurstLength_g,

			EnableBrightness_g	=> EnableBrightness_g,
			BrightPMWDiv_g		=> BrightPMWDiv_g,

			HResolution_g		=> HResolution_g,
			HSyncPulseWidth_g	=> HSyncPulseWidth_g,
			HSyncPulsePol_g		=> HSyncPulsePol_g,
			HFrontPorchLength_g	=> HFrontPorchLength_g,
			HBackPorchLength_g	=> HBackPorchLength_g,

			VResolution_g		=> VResolution_g,
			VSyncPulseWidth_g	=> VSyncPulseWidth_g,
			VSyncPulsePol_g		=> VSyncPulsePol_g,
			VFrontPorchLength_g	=> VFrontPorchLength_g,
			VBackPorchLength_g	=> VBackPorchLength_g,

			BlackFramesPU_g		=> BlackFramesPU_g,
			BlackFramesPD_g		=> BlackFramesPD_g,

			RdFifoDepth_g		=> RdFifoDepth_g,
			RdFifoFillLevel_g	=> RdFifoFillLevel_g,

			UseAsyncFifos_g		=> UseAsyncFifos_g,
			DispClkFreq_g		=> DispClkFreq_g,
			M_AXI_ClkFreq_g		=> M_AXI_ClkFreq_g
		)
		port map (
			-- global signals
			Clk					=> Clk,
			Rst					=> Rst,

			-- Display signal
			Disp_Clk			=> open,
			Disp_Rdy			=> Disp_Rdy,
			Disp_Ena			=> Disp_Ena,
			Disp_VSync			=> Disp_VSync,
			Disp_HSync			=> Disp_HSync,
			Disp_Bright			=> Disp_Bright,
			Disp_DTMG			=> Disp_DTMG,
			Disp_Data			=> Disp_Data,

			-- AXI4lite slave interface
			S_AXI_ACLK			=> S_AXI_ACLK,
			S_AXI_ARESETN		=> S_AXI_ARESETN,
			S_AXI_AWADDR		=> S_AXI_AWADDR,
			S_AXI_AWVALID		=> S_AXI_AWVALID,
			S_AXI_WDATA			=> S_AXI_WDATA,
			S_AXI_WSTRB			=> S_AXI_WSTRB,
			S_AXI_WVALID		=> S_AXI_WVALID,
			S_AXI_BREADY		=> S_AXI_BREADY,
			S_AXI_ARADDR		=> S_AXI_ARADDR,
			S_AXI_ARVALID		=> S_AXI_ARVALID,
			S_AXI_RREADY		=> S_AXI_RREADY,
			S_AXI_ARREADY		=> S_AXI_ARREADY,
			S_AXI_RDATA			=> S_AXI_RDATA,
			S_AXI_RRESP			=> S_AXI_RRESP,
			S_AXI_RVALID		=> S_AXI_RVALID,
			S_AXI_WREADY		=> S_AXI_WREADY,
			S_AXI_BRESP			=> S_AXI_BRESP,
			S_AXI_BVALID		=> S_AXI_BVALID,
			S_AXI_AWREADY		=> S_AXI_AWREADY,
			Irpt_N				=> Irpt_N,

			-- AXI4 master interface (display)
			M_AXI_ACLK			=> M_AXI_ACLK,
			M_AXI_ARESETN		=> M_AXI_ARESETN,
			M_AXI_AWADDR		=> M_AXI_AWADDR,
			M_AXI_AWVALID		=> M_AXI_AWVALID,
			M_AXI_AWREADY		=> M_AXI_AWREADY,
			M_AXI_AWLEN			=> M_AXI_AWLEN,
			M_AXI_AWSIZE		=> M_AXI_AWSIZE,
			M_AXI_AWBURST		=> M_AXI_AWBURST,
			M_AXI_AWLOCK		=> M_AXI_AWLOCK,
			M_AXI_AWCACHE		=> M_AXI_AWCACHE,
			M_AXI_AWPROT		=> M_AXI_AWPROT,
			M_AXI_WDATA			=> M_AXI_WDATA,
			M_AXI_WSTRB			=> M_AXI_WSTRB,
			M_AXI_WVALID		=> M_AXI_WVALID,
			M_AXI_WREADY		=> M_AXI_WREADY,
			M_AXI_WLAST			=> M_AXI_WLAST,
			M_AXI_BRESP			=> M_AXI_BRESP,
			M_AXI_BVALID		=> M_AXI_BVALID,
			M_AXI_BREADY		=> M_AXI_BREADY,
			M_AXI_ARADDR		=> M_AXI_ARADDR,
			M_AXI_ARVALID		=> M_AXI_ARVALID,
			M_AXI_ARREADY		=> M_AXI_ARREADY,
			M_AXI_ARLEN			=> M_AXI_ARLEN,
			M_AXI_ARSIZE		=> M_AXI_ARSIZE,
			M_AXI_ARBURST		=> M_AXI_ARBURST,
			M_AXI_ARLOCK		=> M_AXI_ARLOCK,
			M_AXI_ARCACHE		=> M_AXI_ARCACHE,
			M_AXI_ARPROT		=> M_AXI_ARPROT,
			M_AXI_RDATA			=> M_AXI_RDATA,
			M_AXI_RVALID		=> M_AXI_RVALID,
			M_AXI_RREADY		=> M_AXI_RREADY,
			M_AXI_RRESP			=> M_AXI_RRESP,
			M_AXI_RLAST			=> M_AXI_RLAST,
			
			-- AXI4 master interface (2D accelerator)
			M2_AXI_ACLK			=> M2_AXI_ACLK,
			M2_AXI_ARESETN		=> M2_AXI_ARESETN,
			M2_AXI_AWADDR		=> M2_AXI_AWADDR,
			M2_AXI_AWVALID		=> M2_AXI_AWVALID,
			M2_AXI_AWREADY		=> M2_AXI_AWREADY,
			M2_AXI_AWLEN		=> M2_AXI_AWLEN,
			M2_AXI_AWSIZE		=> M2_AXI_AWSIZE,
			M2_AXI_AWBURST		=> M2_AXI_AWBURST,
			M2_AXI_AWLOCK		=> M2_AXI_AWLOCK,
			M2_AXI_AWCACHE		=> M2_AXI_AWCACHE,
			M2_AXI_AWPROT		=> M2_AXI_AWPROT,
			M2_AXI_WDATA		=> M2_AXI_WDATA,
			M2_AXI_WSTRB		=> M2_AXI_WSTRB,
			M2_AXI_WVALID		=> M2_AXI_WVALID,
			M2_AXI_WREADY		=> M2_AXI_WREADY,
			M2_AXI_WLAST		=> M2_AXI_WLAST,
			M2_AXI_BRESP		=> M2_AXI_BRESP,
			M2_AXI_BVALID		=> M2_AXI_BVALID,
			M2_AXI_BREADY		=> M2_AXI_BREADY,
			M2_AXI_ARADDR		=> M2_AXI_ARADDR,
			M2_AXI_ARVALID		=> M2_AXI_ARVALID,
			M2_AXI_ARREADY		=> M2_AXI_ARREADY,
			M2_AXI_ARLEN		=> M2_AXI_ARLEN,
			M2_AXI_ARSIZE		=> M2_AXI_ARSIZE,
			M2_AXI_ARBURST		=> M2_AXI_ARBURST,
			M2_AXI_ARLOCK		=> M2_AXI_ARLOCK,
			M2_AXI_ARCACHE		=> M2_AXI_ARCACHE,
			M2_AXI_ARPROT		=> M2_AXI_ARPROT,
			M2_AXI_RDATA		=> M2_AXI_RDATA,
			M2_AXI_RVALID		=> M2_AXI_RVALID,
			M2_AXI_RREADY		=> M2_AXI_RREADY,
			M2_AXI_RRESP		=> M2_AXI_RRESP,
			M2_AXI_RLAST		=> M2_AXI_RLAST
		);
	
	if_cw24	: if ColorWidth_g = 24 generate
		Dvi_Data <= Disp_Data;
	end generate;
	if_cw16	: if ColorWidth_g < 24 generate -- map 16 bit to 24 bit
		Dvi_Data(7  downto 0)  <= Disp_Data(4 downto 0) & Disp_Data(0) & Disp_Data(0) & Disp_Data(0);
		Dvi_Data(15 downto 8)  <= Disp_Data(10 downto 5) & Disp_Data(5) & Disp_Data(5);
		Dvi_Data(23 downto 16) <= Disp_Data(15 downto 11) & Disp_Data(11) & Disp_Data(11) & Disp_Data(11);
	end generate;

	-----------------------------------------------------------------------------------------------
	-- DVI output core
	-----------------------------------------------------------------------------------------------

	i_dvi : entity lib_en_udc_axi_2d_dvi_xil_x7.en_udc_dvi_xil_x7
		generic map (
			ClkFreqHz_g			=> DispClkFreq_g
		)
		port map (
			---------------------------------------------
			-- clock and reset
			---------------------------------------------
			Clk					=> Clk,
			Rst					=> Rst,
			---------------------------------------------
			-- Video data input
			---------------------------------------------
			InData				=> Dvi_Data,
			InVsync				=> Disp_VSync,
			InHsync				=> Disp_HSync,
			InDe				=> Disp_DTMG,
			InData_Rdy			=> Disp_Rdy,
			---------------------------------------------
			-- HDMI/DVI output
			---------------------------------------------
			OutDviClk_p			=> DviClk_p,
			OutDviClk_n			=> DviClk_n,
			OutDviData_p		=> DviData_p,
			OutDviData_n		=> DviData_n	
		);



end;
