---------------------------------------------------------------------------------------------------
--	Project				: Mars ZX3 SDSoC reference design
--	File description	: Top level file for Mars ZX3-PM3 with DVI output
--	File name			: mars_zx3_pm3_wrapper.vhd
--	Author				: Christoph Glattfelder
---------------------------------------------------------------------------------------------------
--	Copyright (c) 2016 by Enclustra GmbH, Switzerland
--	All rights reserved.
---------------------------------------------------------------------------------------------------
--	Notes:
--
---------------------------------------------------------------------------------------------------


---------------------------------------------------------------------------------------------------
-- libraries
---------------------------------------------------------------------------------------------------

library ieee;
	use ieee.std_logic_1164.all;
	use ieee.numeric_std.all;

library unisim;
	use unisim.vcomponents.all;
	
---------------------------------------------------------------------------------------------------
-- entity declaration
---------------------------------------------------------------------------------------------------    
	
entity mars_zx3_eb1_wrapper is
  port (
		DDR_addr			: inout std_logic_vector ( 14 downto 0 );
		DDR_ba				: inout std_logic_vector ( 2 downto 0 );
		DDR_cas_n			: inout std_logic;
		DDR_ck_n			: inout std_logic;
		DDR_ck_p			: inout std_logic;
		DDR_cke				: inout std_logic;
		DDR_cs_n			: inout std_logic;
		DDR_dm				: inout std_logic_vector ( 3 downto 0 );
		DDR_dq				: inout std_logic_vector ( 31 downto 0 );
		DDR_dqs_n			: inout std_logic_vector ( 3 downto 0 );
		DDR_dqs_p			: inout std_logic_vector ( 3 downto 0 );
		DDR_odt				: inout std_logic;
		DDR_ras_n			: inout std_logic;
		DDR_reset_n			: inout std_logic;
		DDR_we_n			: inout std_logic;
		
		FIXED_IO_ddr_vrn	: inout std_logic;
		FIXED_IO_ddr_vrp	: inout std_logic;
		FIXED_IO_mio		: inout std_logic_vector ( 53 downto 0 );
		FIXED_IO_ps_clk		: inout std_logic;
		FIXED_IO_ps_porb	: inout std_logic;
		FIXED_IO_ps_srstb	: inout std_logic;

		Eth_Rst_N			: inout	std_logic;
		Usb_Rst_N			: inout	std_logic;

		UART0_TX			: out 	std_logic; 
		UART0_RX 			: in 	std_logic;

		I2C0_SDA			: inout std_logic; 
		I2C0_SCL			: inout std_logic; 
		I2C0_INT_N			: in 	std_logic; 

		Led_N 				: out	std_logic_vector(3 downto 0);
		
		Dvi_clk_n			: out 	std_logic;
		Dvi_clk_p			: out 	std_logic;		
		Dvi_data_n			: out 	std_logic_vector ( 2 downto 0 );
		Dvi_data_p			: out 	std_logic_vector ( 2 downto 0 );
		
		-- unused pins, set to high impedance
		Vref0				: inout std_logic; 
		Vref1				: inout std_logic; 
		CLK33				: inout std_logic;
		NAND_WP				: inout std_logic;
		PWR_GOOD_R			: inout std_logic;
        DDR3_VSEL           : inout std_logic;
		ETH_Link			: inout std_logic;
		ETH_MDC				: inout std_logic; 
		ETH_MDIO			: inout std_logic; 
		ETH_RX_CLK			: inout std_logic; 
		ETH_RX_CTL			: inout std_logic; 
		ETH_RXD				: inout std_logic_vector(3 downto 0);
		ETH_TX_CLK			: inout std_logic; 
		ETH_TX_CTL			: inout std_logic; 
		ETH_TXD				: inout std_logic_vector(3 downto 0);
		MIO40				: inout std_logic;
        MIO41               : inout std_logic;
        MIO42               : inout std_logic;
        MIO43               : inout std_logic;
        MIO44               : inout std_logic;
        MIO45               : inout std_logic;
--        MIO46               : inout std_logic;
--        MIO47               : inout std_logic;
        MIO48               : inout std_logic;
        MIO49               : inout std_logic;
        MIO50               : inout std_logic;
        MIO51               : inout std_logic
      );
end mars_zx3_eb1_wrapper;

---------------------------------------------------------------------------------------------------
-- architecture implementation
---------------------------------------------------------------------------------------------------

architecture STRUCTURE of mars_zx3_eb1_wrapper is

	------------------------------------------------------------------------------------------------
	-- components
	------------------------------------------------------------------------------------------------

	component mars_zx3 is
		port (
			DDR_cas_n			: inout std_logic;
			DDR_cke				: inout std_logic;
			DDR_ck_n			: inout std_logic;
			DDR_ck_p			: inout std_logic;
			DDR_cs_n			: inout std_logic;
			DDR_reset_n			: inout std_logic;
			DDR_odt				: inout std_logic;
			DDR_ras_n			: inout std_logic;
			DDR_we_n			: inout std_logic;
			DDR_ba				: inout std_logic_vector ( 2 downto 0 );
			DDR_addr			: inout std_logic_vector ( 14 downto 0 );
			DDR_dm				: inout std_logic_vector ( 3 downto 0 );
			DDR_dq				: inout std_logic_vector ( 31 downto 0 );
			DDR_dqs_n			: inout std_logic_vector ( 3 downto 0 );
			DDR_dqs_p			: inout std_logic_vector ( 3 downto 0 );
			FIXED_IO_mio		: inout std_logic_vector ( 53 downto 0 );
			FIXED_IO_ddr_vrn	: inout std_logic;
			FIXED_IO_ddr_vrp	: inout std_logic;
			FIXED_IO_ps_srstb	: inout std_logic;
			FIXED_IO_ps_clk		: inout std_logic;
			FIXED_IO_ps_porb	: inout std_logic;
			UART_0_txd			: out	std_logic;
			UART_0_rxd			: in	std_logic;
			IIC_0_sda_i			: in	std_logic;
			IIC_0_sda_o			: out	std_logic;
			IIC_0_sda_t			: out	std_logic;
			IIC_0_scl_i			: in	std_logic;
			IIC_0_scl_o			: out	std_logic;
			IIC_0_scl_t			: out	std_logic;
			RESET_N				: out	std_logic;
			FCLK_CLK1			: out	std_logic;
			Dvi_clk_n			: out 	std_logic;
			Dvi_clk_p			: out 	std_logic;
			Dvi_data_n			: out 	std_logic_vector ( 2 downto 0 );
			Dvi_data_p			: out 	std_logic_vector ( 2 downto 0 )
		);
	end component mars_zx3;
 
	------------------------------------------------------------------------------------------------
	-- internal signals
	------------------------------------------------------------------------------------------------
    
	signal IIC_0_sda_i 		: std_logic;
	signal IIC_0_sda_o 		: std_logic;
	signal IIC_0_sda_t 		: std_logic;
	signal IIC_0_scl_i 		: std_logic;
	signal IIC_0_scl_o 		: std_logic;
	signal IIC_0_scl_t 		: std_logic;

	signal Clk				: std_logic;
	signal Rst				: std_logic := '0';
	signal Rst_N 			: std_logic := '1';
	signal ETH_RST			: std_logic := '0';
	
	signal RstCnt			: unsigned (15 downto 0) := (others => '0'); -- 1ms reset for Ethernet PHY
	signal LedCount			: unsigned (25 downto 0);
	
begin

	mars_zx3_i: component mars_zx3
    	port map (
			DDR_addr			=> DDR_addr,
			DDR_ba				=> DDR_ba,
			DDR_cas_n			=> DDR_cas_n,
			DDR_ck_n			=> DDR_ck_n,
			DDR_ck_p			=> DDR_ck_p,
			DDR_cke				=> DDR_cke,
			DDR_cs_n			=> DDR_cs_n,
			DDR_dm				=> DDR_dm,
			DDR_dq				=> DDR_dq,
			DDR_dqs_n			=> DDR_dqs_n,
			DDR_dqs_p			=> DDR_dqs_p,
			DDR_odt				=> DDR_odt,
			DDR_ras_n			=> DDR_ras_n,
			DDR_reset_n			=> DDR_reset_n,
			DDR_we_n			=> DDR_we_n,
			FCLK_CLK1			=> Clk,
			FIXED_IO_ddr_vrn	=> FIXED_IO_ddr_vrn,
			FIXED_IO_ddr_vrp	=> FIXED_IO_ddr_vrp,
			FIXED_IO_mio		=> FIXED_IO_mio,
			FIXED_IO_ps_clk		=> FIXED_IO_ps_clk,
			FIXED_IO_ps_porb	=> FIXED_IO_ps_porb,
			FIXED_IO_ps_srstb	=> FIXED_IO_ps_srstb,
			RESET_N				=> Rst_N,
			UART_0_rxd			=> UART0_RX,
			UART_0_txd			=> UART0_TX,
			IIC_0_sda_i 		=> IIC_0_sda_i,
			IIC_0_sda_o  		=> IIC_0_sda_o,
			IIC_0_sda_t  		=> IIC_0_sda_t,
			IIC_0_scl_i  		=> IIC_0_scl_i,
			IIC_0_scl_o  		=> IIC_0_scl_o,
			IIC_0_scl_t  		=> IIC_0_scl_t,
			Dvi_clk_n			=> Dvi_clk_n,
			Dvi_clk_p			=> Dvi_clk_p,
			Dvi_data_n			=> Dvi_data_n,
			Dvi_data_p			=> Dvi_data_p
		);
    
	-- supply voltage for thh DDR3 memory
	DDR3_VSEL       <= '0'; -- 1.35V
--  DDR3_VSEL       <= 'Z'; -- 1.5V
    
	-- tristate buffer
	I2C0_SDA	<= IIC_0_sda_o when IIC_0_sda_t = '0' else 'Z';
	IIC_0_sda_i <= I2C0_SDA;
	I2C0_SCL	<= IIC_0_scl_o when IIC_0_scl_t = '0' else 'Z';
	IIC_0_scl_i <= I2C0_SCL;
    
	------------------------------------------------------------------------------------------------
	--    Clock and Reset
	------------------------------------------------------------------------------------------------ 

	--  reset 1ms reset for Ethernet PHY
	   process (Clk)
	   begin
		if rising_edge (Clk) then
			   if (not RstCnt) = 0 then
				   Rst            <= '0';
			   else
				   Rst            <= '1';
				   RstCnt        <= RstCnt + 1;
			   end if;
		   end if;
	   end process;

	ETH_RST_N <= '0' when Rst = '1' else 'Z';
	USB_RST_N <= '0' when Rst = '1' else 'Z';
        
	------------------------------------------------------------------------------------------------
	-- Blinking LED counter
	------------------------------------------------------------------------------------------------

	process (Clk)
	begin
		if rising_edge (Clk) then
			if Rst = '1' then
				LedCount    <= (others => '0');
			else
				LedCount <= LedCount + 1;
			end if;
		end if;
	end process;


	Led_N(3) <= not LedCount(25);
	Led_N(2) <= '1';
	Led_N(1) <= '1'; 
	Led_N(0) <= '1'; 
    
	------------------------------------------------------------------------------------------------
	-- Unused pins
	------------------------------------------------------------------------------------------------

 	-- system pins, set to high impedance if not used
	Vref0 			<= 'Z';
	Vref1 			<= 'Z';
	CLK33			<= 'Z';
    NAND_WP			<= 'Z';
    PWR_GOOD_R		<= 'Z';
    
    -- shared mio pins, set to high impedance if not used
	MIO40			<= 'Z'; 
    MIO41			<= 'Z'; 
    MIO42			<= 'Z'; 
    MIO43			<= 'Z'; 
    MIO44			<= 'Z'; 
    MIO45			<= 'Z'; 
--    MIO46			<= 'Z';  -- used for uart on Mars EB1
--    MIO47			<= 'Z';  -- used for uart on Mars EB1
    MIO48			<= 'Z'; 
    MIO49			<= 'Z'; 
    MIO50			<= 'Z'; 
    MIO51			<= 'Z';     
	ETH_Link		<= 'Z';	
	ETH_MDC			<= 'Z';
	ETH_MDIO		<= 'Z'; 
	ETH_RX_CLK		<= 'Z'; 
	ETH_RX_CTL		<= 'Z'; 
	ETH_RXD			<= (others => 'Z'); 
	ETH_TX_CLK		<= 'Z'; 
	ETH_TX_CTL		<= 'Z'; 
	ETH_TXD			<= (others => 'Z'); 
    
end STRUCTURE;

----------------------------------------------------------------------------------------------------
-- eof
----------------------------------------------------------------------------------------------------
